"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.plugin = void 0;
const path_1 = __importDefault(require("path"));
const framework_core_1 = require("@cloudbase/framework-core");
class FunctionPlugin extends framework_core_1.Plugin {
    constructor(name, api, inputs) {
        super(name, api, inputs);
        this.name = name;
        this.api = api;
        this.inputs = inputs;
        const config = this.api.projectConfig;
        const DEFAULT_INPUTS = {
            functionRootPath: (config === null || config === void 0 ? void 0 : config.functionRoot) || "cloudfunctions",
            functions: config === null || config === void 0 ? void 0 : config.functions,
            servicePaths: {},
        };
        this.resolvedInputs = resolveInputs(this.inputs, DEFAULT_INPUTS);
        this.functions = this.resolvedInputs.functions;
        this.functionRootPath = path_1.default.isAbsolute(this.resolvedInputs.functionRootPath)
            ? this.resolvedInputs.functionRootPath
            : path_1.default.join(this.api.projectPath, this.resolvedInputs.functionRootPath);
    }
    /**
     * 初始化
     */
    init() {
        return __awaiter(this, void 0, void 0, function* () {
            this.api.logger.debug("FunctionPlugin: init", this.resolvedInputs);
        });
    }
    compile() {
        return __awaiter(this, void 0, void 0, function* () {
            this.api.logger.debug("FunctionPlugin: compile", this.resolvedInputs);
            return {
                Resources: this.functions.reduce((resouces, func) => {
                    resouces[this.toConstantCase(func.name)] = this.functionConfigToSAM(func);
                    return resouces;
                }, {}),
            };
        });
    }
    /**
     * 删除资源
     */
    remove() {
        return __awaiter(this, void 0, void 0, function* () { });
    }
    /**
     * 生成代码
     */
    genCode() {
        return __awaiter(this, void 0, void 0, function* () { });
    }
    /**
     * 构建
     */
    build() {
        return __awaiter(this, void 0, void 0, function* () {
            this.api.logger.debug("FunctionPlugin: build", this.resolvedInputs);
        });
    }
    /**
     * 部署
     */
    deploy() {
        var _a;
        return __awaiter(this, void 0, void 0, function* () {
            this.api.logger.debug("FunctionPlugin: deploy", this.resolvedInputs, this.buildOutput);
            const Function = (_a = this.api.resourceProviders) === null || _a === void 0 ? void 0 : _a.function;
            // 批量部署云函数
            yield Promise.all(this.functions.map((func) => __awaiter(this, void 0, void 0, function* () {
                try {
                    yield Function.createFunction({
                        func,
                        envId: this.api.envId,
                        force: true,
                        functionRootPath: this.functionRootPath,
                    });
                    this.api.logger.info(`${this.api.emoji("🚀")} [${func.name}] 云函数部署成功`);
                }
                catch (e) {
                    this.api.logger.error(`${this.api.emoji("🙅‍♂")} [${func.name}] 函数部署失败`);
                    throw new Error(e.message);
                }
            })));
            // 批量处理云接入
            yield Promise.all(Object.entries(this.resolvedInputs.servicePaths).map(([functionName, servicePath]) => __awaiter(this, void 0, void 0, function* () {
                try {
                    yield this.api.cloudbaseManager.commonService().call({
                        Action: "CreateCloudBaseGWAPI",
                        Param: {
                            ServiceId: this.api.envId,
                            Path: servicePath,
                            Type: 1,
                            Name: functionName,
                        },
                    });
                }
                catch (e) {
                    if (!e.message.includes("api created")) {
                        throw e;
                    }
                }
                let url = `https://${this.api.envId}.service.tcloudbase.com${servicePath}`;
                if (url[url.length - 1] !== "/") {
                    url = url + "/";
                }
                url = this.api.genClickableLink(url);
                this.api.logger.info(`${this.api.emoji("🚀")} 云接入服务发布成功，访问地址: ${url}`);
            })));
            this.api.logger.info(`${this.api.emoji("🚀")} 云函数部署成功`);
        });
    }
    functionConfigToSAM(funcitonConfig) {
        return {
            Type: "CloudBase::Function",
            Properties: {
                Handler: funcitonConfig.handler || "index.main",
                Description: "",
                Runtime: funcitonConfig.runtime,
                FunctionName: funcitonConfig.name,
                MemorySize: funcitonConfig.memory || 128,
                Timeout: funcitonConfig.timeout || 3,
                Environment: funcitonConfig.envVariables,
                VpcConfig: funcitonConfig.vpc,
                HttpPath: this.resolvedInputs.servicePaths[funcitonConfig.name],
            },
        };
    }
    toConstantCase(name) {
        let result = "";
        let lastIsDivide = true;
        for (let i = 0; i < name.length; i++) {
            let letter = name[i];
            if (letter === "-" || letter === "_") {
                lastIsDivide = true;
            }
            else if (lastIsDivide) {
                result += letter.toUpperCase();
                lastIsDivide = false;
            }
            else {
                result += letter.toLowerCase();
                lastIsDivide = false;
            }
        }
        return result;
    }
}
function resolveInputs(inputs, defaultInputs) {
    return Object.assign({}, defaultInputs, inputs);
}
function wait(time) {
    return new Promise((resolve) => {
        setTimeout(resolve, time);
    });
}
exports.plugin = FunctionPlugin;
